@@ -11,18 +11,29 @@
 #'
 #' @export
 #'
-run.idem <- function() {
+imShiny <- function() {
 
     if (!requireNamespace("shiny", quietly = TRUE)) {
         stop("Shiny needed for this function to work. Please install it.",
              call. = FALSE)
     }
 
+    if (!requireNamespace("shinythemes", quietly = TRUE)) {
+        stop("shinythemes needed for this function to work. Please install it.",
+             call. = FALSE)
+    }
+
+    if (!requireNamespace("DT", quietly = TRUE)) {
+        stop("DT needed for this function to work. Please install it.",
+             call. = FALSE)
+    }
+
     appDir <- system.file("shiny", package = "idem")
     if (appDir == "") {
         stop("Could not find Shiny directory. Try re-installing `idem`.",
              call. = FALSE)
     }
 
+
     shiny::runApp(appDir, display.mode = "normal");
 }
